/*
** 	Algorithm to generate lookup tables for random distributions
** 			   The set is [1...N] contained in array x[]
** 			   the probabilities  are  "      "      p[] 
**				(presently NORMAL DISTRIBUTION)
** 				In output one obtains the tables Y/P
** 	       see Knuth's "Art of computer programming" (4.1) vol.2
**  
** (C) E.Onofri, CERN, summer 1986
*/


#include  stdio
#include  math

#define   ST_DEV  5

/* Include the following if you want to check the accuracy 
**							  *
#define   CHECK                                          */

/***********
**  FLIP **    Intechange of contiguous elements 
**********/

void flip(double *p, int *x)      
{ double ptmp;
  int xtmp;
  ptmp= *p; *p= *(p+1); *(p+1)= ptmp;
  xtmp= *x; *x= *(x+1); *(x+1)= xtmp;
} 

/**********
*  SWAP **
*********/

void swap(int *x, int *y, double *p, double *q)  
{ int i;
  double t;
  t = *p; *p = *q; *q = t;
  i = *x; *x = *y; *y = i;
}


/************
**  RESET **	Relocate an element in an already sorted set 
***********/

void reset(double *p, int *x,int n)       
{ double *pmax;      			 
  pmax=p+n-1;
  while(p<pmax && *p>*(p+1)) 
         flip(p++,x++);
}
      	

/**************
**  RESTORE ** 		Restore the heap 
*************/
		
void restore(double *p,int *x,int left,int right)      
{ int m;
  while(left<=right/2) {
	m = 2*left-1;
	if(p[m]<p[m+1] && 2*left<right)
			m++;
	if(p[m]>p[left-1]) {
			swap(x+m,x+left-1,p+m,p+left-1);
			left=m+1;
			}
		else
			left=right;
	}
}


/****************
**   S O R T  **	heap sort 
***************/

void sort(double *p, int *x, int n)  		
{int i;
 for(i=n/2;i>0;i--)
	restore(p,x,i,n);
 while(--n) {
	swap(x,x+n,p,p+n);
	restore(p,x,1,n);
	}
}


/***********
** KNUTH **
**********/

knuth(double *p, double *q, int *x,int *y,int size) 
						       
{	int n=size,k;
	sort(p,x,n);      	              /*  First  sort  according  */
	while(--n) {			     /* to probabilities  	 */
		k=x[0];                     /* then fill up the tables  */
		q[k] = size * p[0];        /*                          */
		y[k] = (x[0] = x[n]);     /*                          */
		p[0] += p[n]-1.0/size;   /*     new entry  	     */
		reset(p,x,n);           /*  restore sorting         */
	}   	                       /* eliminate old 0-th entry */
}

/*************
** M A I N **
************/

main()
{ int    *x,*y,i,j,npts=1,center;
  double *q, *p, *prob;
  double sum, scale, norm=0.0;

  printf("How many points?\n");   /* Prompt for number of entries */
  scanf("%d",&npts);

  center = (npts-1.0)/2.0;

  x = (int *)calloc(npts,sizeof(int));
  y = (int *)calloc(npts,sizeof(int));
  p = (double *)calloc(npts,sizeof(double));
  q = (double *)calloc(npts,sizeof(double));
  scale = pow(2.0*(ST_DEV/(double)npts),2);
                                   /*  Define weights according  **/
			          /**   to  normal distribution  */
  for(i=0;i<npts;++i) {
	x[i] = i;
	y[i] = -1;                 /*  In the final table  Y=-1 and   q=1 */
	q[i] = 1.0;               /** If i is not reached in the process */
	p[i] = exp( -0.5*scale*(double)((i-center)*(i-center)));
	norm += p[i];
      }

  norm = 1./norm;
  for(i=0;i<npts;++i) 
	p[i] *= norm;

#ifdef CHECK
  prob = (double *)calloc(npts,sizeof(double));
  for(i=0;i<npts;++i) 
	prob[i] = p[i];
#endif
  
  knuth(p,q,x,y,npts);   /*  invoke the main routine to build the tables */

  printf("\n\t\t  * * *   Lookup Table  * * *  \n\n");

  for(i=0;i<npts;i+=2)
	printf("%3d: Y=%3d  P=%9.6f \t%3d: Y=%3d P=%9.6f\n",i,y[i],q[i],i+1,y[i+1],q[i+1]);


#ifdef CHECK
  printf("\n\t\t\t * * *    C h e c k    * * *  \n\n");

  for(i=0;i<npts;i++) { sum=q[i];
		     if(y[i] == -1 && q[i] != 1)
				printf("Error at entry %d\n",i);
		     for(j=0;j<npts;++j) 
				if(y[j]==i)
					sum += 1.0-q[j];
		     printf("Prob(%3d)=%10.8f \t %10.8f\n",i,sum/npts,prob[i]); 
		}	
#endif
}
