function Kdemo(Nevents)
% Examples of application of Knuth's alias method to produce samples from
% various probability distributions
% 
% Usage:
% Kdemo(Nevents)
% Defaults: Nevents=1e5 --- should not greater than than 1e5
%
% Source: D.Knuth, The Art of Computer Programming, Vol.2
% See also: Computing in Science and Eng., July issue 1999, paper
% by I.Beichl and F.Sullivan
%
% E.Onofri  2000-2018

if nargin<1,          Nevents=1e5;
elseif(Nevents>1e5),  Nevents=1e5; end

rng('shuffle')

close all
check='go';

figure('windowstyle','docked')

while(check=='go'),
   selection = {'dice throw','diffraction pattern','poisson', ...
	       'gaussian','lorentzian','linear','quit'};
   choice    = menu('Start',selection);
   
   switch choice		
     case 1					% Knuth's example
       result=zeros(3,1);
       disp('Alice, Bruno and Charles are drawing dice for 100 times')
       players={'Alice';'Bruno';'Charles'};
       p=[1 2 3 4 5 6 5 4 3 2 1];       % prob for two dyes
       KLT=kalias(p);
       
       for j=1:100,
          X = krand(3,KLT);
          s = max(X);
          ind = find(X==s);
          result(ind) = result(ind) + 1/length(ind);
       end
       score=max(result);
       win=find(result==score);
       disp(['The winner is ' players{win} ' by ' num2str(score) ' points ']);
       hold off
       subplot(111)
       bar(result);
     case 2			% fringe
       disp('Running <diffraction pattern demo>');
       disp([num2str(Nevents)...
             ' scintillation counts from low energy electron beam']);      
       x = linspace(0,1,128);
       tic;KLTc = krndini('fringe',x,4,0);toc

       subplot(211)
       X=krandc(Nevents,KLTc);

       subplot(212)
       plot(X,rand(size(X)),'k.','Markersize',1)

     case 3					% Poisson
       disp(['Extracting ' num2str(Nevents) 'events from Poisson distribution']);
       p = poisson(32,8);
       n = 0:32;
       KLT = kalias(p);
       tic;X = krand(Nevents,KLT)-1;toc
       
       subplot(111)

       histogram(X,'BinMethod',...
                 'sturges',... %'fd',... % binning Freedman-Diaconis
                 'DisplayStyle','bar',...
                 'FaceAlpha',.6,...%             'FaceColor','none',...
                 'EdgeColor','auto',...
                 'Normalization','probability'); 
       
       
       grid
       hold on
       
       plot(n+1/2,p,'p-','LineWidth',2)
       ax = axis;
       axis([0 max(X) ax(3) ax(4)]);
       
       if ~verLessThan('matlab','9'),
          xticks(0.5:2:20.5);              % put ticks in bin center!
          xticklabels(num2cell(0:2:20));   % name the ticks
       end
       
       disp(['<X>=' num2str(mean(X)) ' +/- ' num2str(std(X))])
       toc
       legend('Knuth','Poisson')
       legend('boxoff')

     case 4					% gaussian
       disp(['Extracting ' num2str(Nevents)...
             ' events from Gaussian distribution']);
       x=linspace(-10,10,200);
       tic;KLTc=krndini('gaussian',x,0,2);toc
       subplot(111)
       X=krandc(Nevents,KLTc);
     
     case 5
       disp(['Extracting ' num2str(Nevents)...
             ' events from Lorentzian distribution']);
       x=linspace(-100,100,400);		% Lorentzian
       tic;KLTc=krndini('lorentz',x,0,20);toc
       subplot(111)
       X=krandc(Nevents,KLTc);
       legend('boxoff')

    case 6
    disp(['Extracting ' num2str(Nevents)...
	  ' events from linear distribution']);
    n=1:100;
    p=n/sum(n);
    tic;KLT=kalias(p);
    X=krand(Nevents,KLT);t1=toc;
    tic;exact=lpdf(Nevents,100);t2=toc;
    subplot(111)
    histogram(X,50,'Normalization','pdf','DisplayStyle','bar',...
              'Facealpha',.2,'BinMethod','integers');
    hold on
    histogram(exact,50,'Normalization','pdf','DisplayStyle','stairs',...
              'LineWidth',2);
    plot([0,n(end)+5],[-1,n(end)+5]/sum(n),'c-.','LineWidth',2)
    title('freq(n)/p(n) ')
    legend('Knuth','ad hoc','linear pdf','location','northwest')
    legend('boxoff')
    grid
    disp(['Time for alias method:',num2str(t1)])
    disp(['Time for direct method:',num2str(t2)])
    disp('In this case direct method is more efficient!')
   otherwise
    check='quit';
    return
  end


end
end
%--------
function   showhist(x,prob,X);
   hold off
   n=length(x);
   histogram(X,n/4,'BinMethod','fd',... % binning Freedman-Diaconis
             'DisplayStyle','bar',...
             'FaceAlpha',.4,...
             'FaceColor','cyan',...
             'EdgeColor','auto',...
             'EdgeAlpha',1,...
             'Normalization','pdf');    % normalization=prob.distr.fnct
             
   hold on
   dx=x(2)-x(1);
   NRAND=length(X);
   plot(x,prob/dx,'r','LineWidth',1.5);   % normalization: sum(p*dx)=1
   hold off
   grid
   %   legend('random sample','probability distribution');
   % legend('boxoff')

end

