function X=krandc(NRAND,KLTc,seed)
% Returns NRAND random numbers distributed according to the density prob 
% defined on the mesh [x1,x2,...,xn] 
% and linearly interpolated to cover the closed interval [x1,xn].
% Data are packed in the structure KLTc (x=KLTc.mesh, Y=KLTc.Y, etc.)
% Usage: 
% X=krandc(NRAND,KLTc[,seed])
% KLTc is provided by krndini.m; it should not be changed until
% a different probability density is needed
% seed is optional: it can be an integer or a string ('auto'),
% in this latter case seed is chosen by reading the system clock.
% Requires: krndini.m, showhist.m
% Source: D.Knuth, The Art of Computer Programming, Vol.2
% See also: Computing in Science and Eng., July issue 1999, paper
% by I.Beichl and F.Sullivan
%
% Example: to display a histogram of 10^5 random scintillation 
% on a screen due to electron interference pattern according 
% to prob(x) given by 'fringe.m':
% 
% >> x=linspace(0,20,100);
% >> KLTc=krndini('fringe',x,8,1.2);
% >> X=krandc(1e5,KLTc,'init');
% E.ONOFRI, 1998-2000 (onofri@pr.infn.it)

if nargin>5,
  if isstr(seed),
    rand('state',sum(clock)*100);
  else
    rand('state',seed);
  end
end

x=KLTc.mesh;
P=KLTc.P;
Y=KLTc.Y;
prob=KLTc.p;

if(length(x)==length(P)+1),
  disp('');
else
  error('check dimensions, Y and P must be n-1 dim if dim(x)=n');
end

N=length(P);

tmp=zeros(1,NRAND);
r=N*rand(1,NRAND);  
j=floor(r);

v=r-j; % Are low digits random enough?
%v=rand(1,NRAND); % if not use this line instead

tmp=j+1;
ind=find(v>P(j+1));
tmp(ind)=Y(j(ind)+1);
X0=x(tmp); % coarse grained distribution

n=length(x);
right=2:n; 
left=right-1;
a=x(right)-x(left);  % Do not assume equally spaced grid for later use

p=(prob(right)+prob(left));
delta=(prob(right)-prob(left))./p; % linearized prob distribution

a=a(tmp);
delta=delta(tmp);

r=rand(1,NRAND);

csi=2*r./(1-delta + sqrt((1-delta).^2+4*delta.*r)); % csi gives a random
                                   % variable in the coarse grained bins 
X=X0+csi.*a;

showhist(x,prob,X);

